<?php

/**
 * Classe PDFMerger
 * Esta classe é responsável por unir vários arquivos PDF em um único arquivo.
 * 
 * Autor: Rafael Pickbrenner, 06/05/2024. Tarefa "479230 - [HCM] - Criação de Rotina para Concatenação (docx ou pdf) da
 * abertura e encerramento do PGR e PCMSO".
 * 
 * A classe **PDFMerger** é usada para unir vários arquivos PDF em um único arquivo. Ela tem métodos para adicionar
 * arquivos PDF individuais ou vários de uma vez, unir os arquivos PDF adicionados em um único arquivo e excluir os
 * arquivos de entrada e saída. A classe usa a biblioteca **Zend_Pdf** para carregar e manipular os arquivos PDF.
 * 
 * Além disso, a classe agora inclui um método para baixar um arquivo PDF de uma URL e adicioná-lo à lista de arquivos
 * a serem unidos. Este método baixa o arquivo PDF, salva-o no diretório raiz e o adiciona à lista.
 * 
 * Para usar a classe:
 * 1. Instanciá-la com o diretório raiz onde os arquivos PDF estão localizados.
 * 2. Adicionar arquivos PDF à lista de arquivos a serem unidos usando os métodos **adicionarPDF**,
 *    **adicionarPDFPorURL** e/ou **adicionarVariosPDFs**.
 * 3. Unir os arquivos PDF em um único arquivo usando o método **unirPDFs**.
 * 4. Se necessário excluir os arquivos de entrada e saída usando os métodos **excluirArquivos** e
 *    **excluirArquivoSaida**, respectivamente.
 * 
 * Exemplo de uso:
 * $merger = new PDFMerger( $this->hcmParameters->getTempDirectory() );
 * 
 * $merger->adicionarPDF( 'arquivo_pdf_1.pdf' );
 * $merger->adicionarPDF( 'arquivo_pdf_2.pdf' );
 * $merger->adicionarPDF( 'arquivo_pdf_3.pdf' );
 * //ou
 * $merger->adicionarVariosPDFs( array( 'arquivo_pdf_1.pdf', 'arquivo_pdf_2.pdf', 'arquivo_pdf_3.pdf' ) );
 * //ou
 * $merger->adicionarPDFPorURL( 'http://exemplo.com/arquivo_pdf.pdf', 'arquivo_pdf_4.pdf' );
 * 
 * $caminhoArquivoMesclado = $merger->unirPDFs( 'arquivo_mesclado.pdf' );
 * 
 * $merger->excluirArquivos();
 * $merger->excluirArquivoSaida();
 */

namespace HCM\Geral\Logic;

define( 'ERRO_ARQUIVO_NAO_ENCONTRADO', 'Erro ao adicionar arquivo PDF: %s não encontrado.'          );
define( 'ERRO_ARQUIVO_NAO_ADICIONADO', 'Erro ao unir PDFs: nenhum arquivo PDF foi adicionado.'      );
define( 'ERRO_BAIXAR_ARQUIVO',         'Erro ao baixar o arquivo PDF da URL: %s'                    );
define( 'ERRO_SALVAR_ARQUIVO',         'Erro ao salvar o arquivo PDF baixado no diretório raiz: %s' );

use Zend_Pdf;
//require_once('tcpdi/tcpdi.php');
require_once('tcpdf/tcpdf.php');
require_once('tcpdf/tcpdi.php');

class PDFMerger {
    
    private $diretorioRaiz; // Diretório onde os arquivos de origem e destino se encontram
    private $arquivoGerado; // Nome do arquivo gerado com os PDFs unidos
    private $arquivosPDF;   // Lista dos arquivos que serão unidos


    /**
     * Construtor da classe
     * Configura o diretório raiz de onde os arquivos PDF serão lidos
     * @param string $diretorioRaiz Caminho para o diretório raiz
     */
    public function __construct( $diretorioRaiz ) {
        $this->diretorioRaiz = $diretorioRaiz . '/';
        $this->arquivosPDF   = [];

        var_dump("classe constr");
        $pdf = new \tcpdf(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

        $pdfFiles = array('/tmp/arquivo1.pdf', '/tmp/arquivo2.pdf');

        // Configurações do documento
        /*$pdf->SetCreator(PDF_CREATOR);
        $pdf->SetAuthor('Autor');
        $pdf->SetTitle('Título');
        $pdf->SetSubject('Assunto');
        $pdf->SetKeywords('TCPDF, PDF, exemplo, PHP');*/

        // Define cabeçalho e rodapé
        /*$pdf->setHeaderData('', PDF_HEADER_LOGO_WIDTH, 'Título do Cabeçalho', 'Cabeçalho');
        $pdf->setFooterData(array(0,64,0), array(0,64,128));*/

        // Configura fontes
        //$pdf->SetFont('helvetica', '', 10);

        foreach($this->files as $file) {
            $pagecount = $this->setSourceFile($file);
            for ($i = 1; $i <= $pagecount; $i++) {
                $tplidx = $this->ImportPage($i);
                $s = $this->getTemplatesize($tplidx);
                $this->AddPage('P', array($s['w'], $s['h']));
                $this->useTemplate($tplidx);
            }
        }

        // Adiciona páginas dos arquivos PDF
        /*foreach ($pdfFiles as $file) {
            $pages = $pdf->load($file);
            for ($i = 1; $i <= $pages; $i++) {
                $tplIdx = $pdf->importPage($i);
                $pdf->AddPage();
                $pdf->useTemplate($tplIdx);
            }
        }*/

        // Saída do PDF
        $pdf->Output('/tmp/aa_merged.pdf', 'F');

        var_dump("classe constr saida");
    }


    /**
     * Adiciona um arquivo PDF à lista de arquivos a serem unidos
     * @param string $nomeArquivoPDF Nome do arquivo PDF (relativo ao diretório raiz)
     */
    public function adicionarPDF( $nomeArquivoPDF ) {
        $arquivoPDF = $this->diretorioRaiz . $nomeArquivoPDF;

        if (file_exists( $arquivoPDF )) {
            $this->arquivosPDF[] = $arquivoPDF;
        } else {
            throw new \Exception( sprintf( ERRO_ARQUIVO_NAO_ENCONTRADO, $nomeArquivoPDF ) );
        }
    }


    /**
     * Adiciona vários arquivos PDF à lista de arquivos a serem unidos
     * @param array $nomesArquivosPDF Array com os nomes dos arquivos PDF (relativos ao diretório raiz)
     */
    public function adicionarVariosPDFs( $nomesArquivosPDF ) {
        foreach( $nomesArquivosPDF as $nomeArquivoPDF ) {
            $this->adicionarPDF( $nomeArquivoPDF );
        }
    }


    /**
     * Baixa um arquivo PDF de uma URL e o adiciona à lista de arquivos a serem unidos
     * @param string $url URL do arquivo PDF
     * @param string $nomeArquivoPDF Nome do arquivo PDF (será salvo no diretório raiz)
     * @throws Exception Se o arquivo não puder ser baixado ou salvo
     */
    public function adicionarPDFPorURL( $url, $nomeArquivoPDF ) {
        $arquivoPDF = $this->diretorioRaiz . $nomeArquivoPDF;

        // Baixa o arquivo PDF
        $conteudoPDF = file_get_contents( $url );
        if ($conteudoPDF === false) {
            throw new \Exception( sprintf( ERRO_BAIXAR_ARQUIVO, $url ) );
        }
        $conteudoPDF = str_replace( '%PDF-1.5', '%PDF-1.3', $conteudoPDF );

        // Salva o arquivo PDF no diretório raiz
        $resultado = file_put_contents( $arquivoPDF, $conteudoPDF );
        if ($resultado === false) {
            throw new \Exception( sprintf( ERRO_SALVAR_ARQUIVO, $arquivoPDF ) );
        }
        chmod( $arquivoPDF, 0755 );

        // Adiciona o arquivo PDF à lista de arquivos a serem unidos, todas as páginas serão unidas
        $this->arquivosPDF[] = array( $arquivoPDF, 'all' );
    }


    /**
     * Une os arquivos PDF e retorna o caminho para o arquivo final
     * @param string $nomeArquivoSaida No do arquivo onde conterá os PDFs unidos
     * @return string Caminho para o arquivo PDF final
     */
    public function unirPDFs( $nomeArquivoSaida ) {
        if( empty( $this->arquivosPDF ) ) {
            throw new \Exception( ERRO_ARQUIVO_NAO_ADICIONADO );
        }

        $fpdi = new \FPDI_with_annots;
        
        foreach( $this->arquivosPDF as $file ) {
            $filename  = $file[0];
            $filepages = $file[1];
            $count     = $fpdi->setSourceFile($filename);
            var_dump("classe saida", $filename);die;
        }




        // Primiro arquivo da lista é o ponto de partida
        $objetoPDFSaida = Zend_Pdf::load( $this->arquivosPDF[0] );
        var_dump("classe"); die;
        // Busca e adiciona os demais arquivos
        for( $i = 1; $i < count( $this->arquivosPDF ); $i++ ) {
            $objetoPDF = Zend_Pdf::load( $this->arquivosPDF[$i] );
            
            // Busca todas as páginas do arquivo
            foreach( $objetoPDF->pages as $page ) {
                $objetoPDFSaida->pages[] = clone $page;
            }
        }

        // Cria o arquivo PDF mesclado. Sobreescreve se necessário
        $this->arquivoGerado = $this->diretorioRaiz . $nomeArquivoSaida;
        if( file_exists( $this->arquivoGerado ) ) {
            unlink( $this->arquivoGerado );
        }
        $objetoPDFSaida->save( $this->arquivoGerado );

        return $this->arquivoGerado;
    }


    /**
     * Exclui todos os arquivos de entrada
     * Este método percorre a lista de arquivos PDF que foram adicionados para união.
     * Após excluir todos os arquivos, a lista de arquivos PDF é esvaziada.
     */
    public function excluirArquivos() {
        // Percorre a lista de arquivos PDF que foram adicionados para merge
        foreach( $this->arquivosPDF as $arquivo ) {
            // Verifica se o arquivo existe e se o arquivo existir, ele é excluído
            if( file_exists( $arquivo ) ) {
                unlink( $arquivo );
            }
        }

        // Esvazia a lista de arquivos PDF
        $this->arquivosPDF = [];
    }


    /**
     * Exclui o arquivo de saída
     * Este método verifica se o arquivo PDF gerado pela união dos arquivos existe no sistema de arquivos.
     * Após excluir o arquivo, a referência para o arquivo gerado é removida.
     */
    public function excluirArquivoSaida() {
        // Verifica se o arquivo gerado está definido
        if( isset( $this->arquivoGerado ) ) {
            // Verifica se o arquivo gerado existe e se o arquivo existir, ele é excluído
            if( file_exists( $this->arquivoGerado ) ) {
                unlink( $this->arquivoGerado );
            }
        }

        // Remove a referência para o arquivo gerado
        unset( $this->arquivoGerado );
    }

}

?>
