<?php

namespace HCM\PTO\Services;

use Doctrine\ORM\EntityManager;

use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Util;
use HCM\Util\Messages;
use HCM\Util\DateUtil;

use HCM\PTO\Services\MarcacaoPontoService;
use HCM\PTO\Services\ApuracaoFrequenciaService;
use HCM\PTO\Repository\OcorrenciaFrequenciaRepository;

class OcorrenciaFrequenciaService {
    
    // ID de status da Programação de ocorrência
    protected const IDPROGOCORCONFIRMADA_PENDENTE   = 'P';
    protected const IDPROGOCORCONFIRMADA_CONFIRMADA = 'S';
    protected const IDPROGOCORCONFIRMADA_REJEITADA  = 'N';
    
    // Padrões de nome de arquivo
    protected const OCORRENCIA_DOWNLOAD_FILE_NAME           = 'DOCUMENTO_OCORRENCIA';
    protected const OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR = '_';
    
    // ID's de dia de semana
    const IDDIASSEMANA_DEFAULT = '1111111';
    
    // Tipos de ocorrência
    protected const NRTIPOPROGOCORRENCIA_ALMOCO                 = 20;
    protected const NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG    = 26;
    
    // Horas a projetar, dentro das ocorrências
    const HRPROJETARINTERVALONPROG_INT_NAO_PROG     = '0015';
    const HRPROJETARINTERVALONPROG_ALMOCO           = '0100';
    
    // ID's de horario
    const           IDTIPOHORARIO_FOLGA     = 'F';
    const           IDTIPOHORARIO_NORMAL    = 'N';
    
    const TIPOPROG_AUSENC_COMP_SE_CRED          = 18;
    const TIPOPROG_AUSENCIA_COMPENSAR           = 2;
    const TIPOPROG_AUSENC_COMP_ATRASOS_SOMENTE  = 27;
    const TIPOPROG_HORAS_FOLGAR                 = 3;
    const TIPOPROG_HORAS_FOLGAR_ACRESC          = 12;
    const TIPOPROG_QUITA_CRED_ACUMULADO         = 8;
    const TIPOPROG_QUITA_SALDO_OU_DEB           = 7;
    const TIPOPROG_QUITA_DEBITO                 = 9;
    const TIPOPROG_COMPLEMENTA_TRABALHO         = 21;
    
    // ID's de tipo de calendario
    const IDTIPODIACALENDARIO_FERIADO   = 'FERIADO';
    const IDTIPODIACALENDARIO_NORMAL    = null;
    
    // Valores disponíveis para o campo IDPROGOCORCONFIRMADA
    protected const AVAILABLE_IDPROGOCORCONFIRMADA  = array(self::IDPROGOCORCONFIRMADA_PENDENTE, self::IDPROGOCORCONFIRMADA_CONFIRMADA, self::IDPROGOCORCONFIRMADA_REJEITADA);
    
    protected const AUTO_FLUSH_SINGLE_REG   = true;
    protected const AUTO_FLUSH_MULTIPLE_REG = false;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, MarcacaoPontoService $marcacaoPontoService, OcorrenciaFrequenciaRepository $ocorrenciaFrequenciaRepository) {
        $this->entityManager                    = $entityManager;
        $this->hcmParameters                    = $hcmParameters;
        $this->marcacaoPontoService             = $marcacaoPontoService;
        $this->ocorrenciaFrequenciaRepository   = $ocorrenciaFrequenciaRepository;
        $this->ptoProgOcorrenciaRepository      = Repositories::PTO_PROGOCORRENCIA;
    }
    
    /**
     * Rotina responsável pela alteração de status de ocorrências de frequências (tabela PTO_PROGOCORRENCIA). Disponíveis: 'Pendente', 'Confirmada' e 'Rejeitada'.
     * 
     *  @param $nrprogocorrencia
     *  @param $idprogocorconfirmada
     *  @param $dsobservacaorecusa
     */
    public function alteraStatusOcorrencia($nrprogocorrencia, $idprogocorconfirmada, $dsobservacaorecusa = null) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        // Valida se o status existe
        $idProgOcorNaoExiste = !in_array($idprogocorconfirmada, self::AVAILABLE_IDPROGOCORCONFIRMADA);
        if ($idProgOcorNaoExiste) {
            return false;
        }
        
        // Busca a ocorrência
        $ptoProgOcorrencia = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array('nrorg' => $nrOrg, 'nrprogocorrencia' => $nrprogocorrencia));
        if (empty($ptoProgOcorrencia)) {
            return false;
        }
        
        // Altera o status
        $ptoProgOcorrencia->setIdprogocorconfirmada($idprogocorconfirmada);

        // Altera a descrção de recusa em caso de rejeição da ocorrência. Se não, apaga a descrição.
        if ($idprogocorconfirmada === self::IDPROGOCORCONFIRMADA_REJEITADA) {
            $ptoProgOcorrencia->setDsobservacaorecusa($dsobservacaorecusa);
        } else {
            $ptoProgOcorrencia->setDsobservacaorecusa(null);
        }
        
        $this->entityManager->flush();
        
        return true;
    }
    
    /**
     *  Rotina responsável pela donwload dos documentos associados à ocorrências de frequências (tabela PTO_PROGOCORRENCIA)
     * 
     *  @param $nrprogocorrencia
     * 
     *  @return $filePath
     */
    public function downloadArquivosOcorrencia($nrprogocorrencia) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        // Busca a ocorrência
        $ptoProgOcorrencia = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array('nrorg' => $nrOrg, 'nrprogocorrencia' => $nrprogocorrencia));
        if (empty($ptoProgOcorrencia)) {
            throw new \Exception(Messages::HCM07018_PTO_OCORR_NOT_FOUND);
        }
        
        $dsPathOcorrencia = $ptoProgOcorrencia->getDspath();
        
        if (empty($dsPathOcorrencia)) {
            throw new \Exception(Messages::HCM07018_PTO_OCORR_NO_FILE);
        }
        
        $outputFileName = $this->generateDownloadedOcorrenciaFileName($nrprogocorrencia);
        $filePath = $this->util->getNewFileServerFile($dsPathOcorrencia, $outputFileName);
        
        return $filePath;
    }
    
    /**
     *  Rotina responsável por gerar o nome do arquivo relacionado a ocorrência que será baixado a partir dos parâmetros OCORRENCIA_DOWNLOAD_FILE_NAME, OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR, $nrprogocorrencia e a data atual
     * 
     *  @param $nrprogocorrencia
     * 
     *  @return $fileName
     */
    public function generateDownloadedOcorrenciaFileName($nrprogocorrencia) {
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $fileDate = date("YmdHis");
        
        $fileName = self::OCORRENCIA_DOWNLOAD_FILE_NAME . self::OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR . $nrOrg . self::OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR . $nrprogocorrencia . self::OCORRENCIA_DOWNLOAD_FILE_NAME_SEPARATOR . $fileDate;
        
        return $fileName;
    }
    
    /**
     *  Rotina responsável por receber um array de arrays associativos de nrvinculom e nmmarcacaoponto e inserir ocorrências de almoço com base nesses dados
     * 
     *  @param  $dados | e.g. [['NRMARCACAOPONTO' = 1, 'NRVINCULOM' = 2], ['NRMARCACAOPONTO' = 3, 'NRVINCULOM' = 4]]
     * 
     *  @return Bool
     */
    public function geraOcorrenciasAlmoco($dados) {
        if (!is_array($dados)) {
            return false;
        }
        
        $this->entityManager->beginTransaction();
        
        try {
            foreach ($dados as $item) {
                $this->geraOcorrenciaAlmoco($item['NRVINCULOM'], $item['NRMARCACAOPONTO'], self::AUTO_FLUSH_MULTIPLE_REG);
            }
        
        } catch (\Exception $e) {
            $this->entityManager->rollBack();    
            throw $e;
        }
        
        $this->entityManager->flush();
        $this->entityManager->commit();
        return true;
    }
    
    /**
     *  Rotina responsável por realizar o cadastro de ocorrência de almoço, relacionando-a com uma determinada marcação
     * 
     *  @param  $nrvinculom
     *  @param  $nrmarcacaoponto
     *  @param  $$autoFlush         | AUTO_FLUSH_SINGLE_REG
     * 
     *  @return Bool
     *  @throws \Exception
     */
    public function geraOcorrenciaAlmoco($nrvinculom, $nrmarcacaoponto, $autoFlush = self::AUTO_FLUSH_SINGLE_REG) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $cdoperador = $this->hcmParameters->getCdoperador();
        $dtcompetencia = $this->hcmParameters->getDtCompetencia();
        
        // Verifica se já existe ocorrência de almoço relacionada à marcação de ponto
        $ocorrenciaExistente = $this->entityManager->getRepository(Repositories::PTO_PROGOCORRENCIA)->findOneBy(array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom, 'nrmarcacaoponto' => $nrmarcacaoponto));
        if (!empty($ocorrenciaExistente)) {
            throw new \Exception(Messages::HCM07003_PTO_OCORR_ALMOCO_ALREADY_EXISTS);
        }
        
        // Obtem informações da marcação
        $marcacaoPonto = $this->entityManager->getRepository(Repositories::GPE_MARCACAOPONTO)->findOneBy(array('nrorg' => $nrorg, 'nrmarcacaoponto' => $nrmarcacaoponto, 'nrvinculom' => $nrvinculom));
        if (empty($marcacaoPonto)) {
            throw new \Exception(Messages::HCM07003_PTO_MARC_NOT_FOUND);
        }
        
        $dtmarcacaoponto = $marcacaoPonto->getDthrmarcacaoponto();
        if (empty($dtmarcacaoponto)) {
            throw new \Exception(Messages::HCM07003_PTO_MARC_DTHRMARC_NOT_EXIST);
        }
        
        $dtmarcacaoponto = DateUtil::truncateData($dtmarcacaoponto);
        $dtRealMarcacao = $this->marcacaoPontoService->getDataApuracaoMarcacao($dtmarcacaoponto, $marcacaoPonto->getIddiaocorrencia());

        $dtmarcacaopontoString = !empty($dtmarcacaoponto) ? $dtmarcacaoponto->format(DateUtil::FORMATO_DATA) : $dtmarcacaoponto;
        $dtRealMarcacaoString = !empty($dtRealMarcacao) ? $dtRealMarcacao->format(DateUtil::FORMATO_DATA) : $dtRealMarcacao;
        
        // Obtem dados do horário do vínculo para o dia da marcação
        $dadosHorario = $this->marcacaoPontoService->getDadosHorarioVinculo($nrorg, $nrvinculom, $dtRealMarcacaoString, $dtcompetencia);
        if (empty($dadosHorario)) {
            throw new \Exception(Messages::HCM07003_PTO_DADOS_HORARIO_NOT_FOUND);
        }
        
        // Obtém o tipo de ocorrência correto de acordo com a jornada de trabalho do vínculo
        $nrtipoprogocorrencia = $this->getTipoOcorrenciaAlmocoPorJornada($nrvinculom, $dtRealMarcacaoString, $dadosHorario['IDTIPOHORARIO']);
        $hrprojetarintervalonprog = $this->getHrptojetainteravaloByTipoOcorrencia($nrvinculom, $dtRealMarcacaoString, $nrtipoprogocorrencia);
        
        $novaOcorrencia = new $this->ptoProgOcorrenciaRepository;
        $novaOcorrencia->setNrorg($nrorg);
        $novaOcorrencia->setIdprogocorconfirmada(self::IDPROGOCORCONFIRMADA_CONFIRMADA);
        $novaOcorrencia->setNrvinculom($nrvinculom);
        $novaOcorrencia->setDtinicioprogocor($dtRealMarcacao);
        $novaOcorrencia->setDtfimprogocor($dtRealMarcacao);
        $novaOcorrencia->setIddiassemana(self::IDDIASSEMANA_DEFAULT);
        $novaOcorrencia->setHrprojetarintervalonprog($hrprojetarintervalonprog);
        
        $novaOcorrencia->setNrhordiam($dadosHorario['NRHORDIAM']);
        $novaOcorrencia->setNrintervalohorario($dadosHorario['NRINTERVALOHORARIO']);
        $novaOcorrencia->setIdtipohorario($dadosHorario['IDTIPOHORARIO']);
        
        $novaOcorrencia->setNrtipoprogocorrencia($nrtipoprogocorrencia);
        $novaOcorrencia->setNrprogocorrencia(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 12, true));
        $novaOcorrencia->setNrmarcacaoponto($nrmarcacaoponto);
        $novaOcorrencia->setDtinclusao(DateUtil::getDataAtual());
        $novaOcorrencia->setCdoperinclusao($cdoperador);
        $novaOcorrencia->setNrorginclusao($nrorg);
        $novaOcorrencia->setDtultatu(DateUtil::getDataAtual());
        $novaOcorrencia->setCdoperultatu($cdoperador);
        $novaOcorrencia->setNrorgultatu($nrorg);
        
        $this->entityManager->persist($novaOcorrencia);
        
        if ($autoFlush) {
            $this->entityManager->flush();
        }
        
        return true;
    }
    
    /**
     *  A partir de parametros definidos em ApuracaoService, determina qual é o tipo de ocorrência de acordo com o tempo trabalhado. Em uma jornada com horas trabalhadas entre 04:00h e 06:00, o intervalo de 
     * 15 minutos é obrigatório. Nesse caso, o tipo de intervalo é 26. Para os demais casos, é 20.
     * 
     *  @param  $nrVinculo,
     *  @param  $dia
     * 
     *  @return NRTIPOOCORRENCIA | O Nr. Tipo Ocorrência relativo ao tempo trabalhado
     * 
     */
    public function getTipoOcorrenciaAlmocoPorJornada($nrVinculo, $dia, $idtipohorario = null) { 
        // Ocorrencia de almoço convencional
        $defaultReturn  = self::NRTIPOPROGOCORRENCIA_ALMOCO;
        $nrOrg          = $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        
        // Busca pelo IDTIPOHORARIO do dia em questão, caso não tenha sido informado
        if (empty($idtipohorario)) {
            $dadosHorarioVinculo = $this->marcacaoPontoService->getDadosHorarioVinculo($nrVinculo, $nrOrg, $dia);
            if (!empty($dadosHorarioVinculo)) {
                $idtipohorario = $dadosHorarioVinculo['IDTIPOHORARIO'];
            }
        }
        
        // Verifica se o dia é folga. Caso o dia seja folga, é retornado o valor de NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG
        if ($idtipohorario === self::IDTIPOHORARIO_FOLGA) {
            return self::NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG;
        }
        
        // Checa o tempo trabalhado
        $tempoTrabalhado = $this->marcacaoPontoService->calculateHorasTrabalhadas($nrVinculo, $dia);
        
        if (empty($tempoTrabalhado)) {
            return $defaultReturn;
        }
        
        $isTempoTrabalhoFaixaIntervalo15 = $tempoTrabalhado > ApuracaoFrequenciaService::TEMPO_JORNADA_INTERVALO_15_TEMPO_MIN && $tempoTrabalhado <= ApuracaoFrequenciaService::TEMPO_JORNADA_INTERVALO_15_TEMPO_MAX;
        // var_dump($tempoTrabalhado, ApuracaoFrequenciaService::TEMPO_JORNADA_INTERVALO_15_TEMPO_MIN, ApuracaoFrequenciaService::TEMPO_JORNADA_INTERVALO_15_TEMPO_MAX, $isTempoTrabalhoFaixaIntervalo15);die;
        if ($isTempoTrabalhoFaixaIntervalo15) {
            // Ocorrencia de intervalo não programado
            return self::NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG;
        } 
        
        return self::NRTIPOPROGOCORRENCIA_ALMOCO;
    }
    
    /**
     *  Rotina que identifica o horário a ser projetado a partir do tipo de ocorrência informado. Caso o tipo de ocorrência seja 26, o valor das horas projetadas é 0015. Para os demais valores, null
     * é retornado.
     * 
     *  @param  string  $nrVinculo
     *  @param  string  $dia
     *  @param  string  $nrTipoOcorrencia
     * 
     *  @return bool
     * 
     */
    public function getHrptojetainteravaloByTipoOcorrencia($nrVinculo, $dia, $nrTipoOcorrencia) {
        switch ($nrTipoOcorrencia) {
            case self::NRTIPOPROGOCORRENCIA_ALMOCO_INT_NAO_PROG:
                // Verifica se o dia atual é folga para o vínculo
                $tempoTrabalhado = $this->marcacaoPontoService->calculateHorasTrabalhadas($nrVinculo, $dia);

                // Fazer valer os valores definidos nos horários?
                if ($tempoTrabalhado > ApuracaoFrequenciaService::APUR_TEMPO_MIN_SEM_ALMOCO && $tempoTrabalhado <= ApuracaoFrequenciaService::APUR_TEMPO_MAXIMO_SEM_ALMOCO) {
                    return self::HRPROJETARINTERVALONPROG_INT_NAO_PROG;
                } else if ($tempoTrabalhado > ApuracaoFrequenciaService::APUR_TEMPO_MAXIMO_SEM_ALMOCO) {
                    return self::HRPROJETARINTERVALONPROG_ALMOCO;
                }

                return null;
                break;
            
            case self::NRTIPOPROGOCORRENCIA_ALMOCO;
                return null;
                break;
            
            default:
                return null;
        }
    }
    
    public function setOcorrencia($params) {
        $ocorrencia = new $this->ptoProgOcorrenciaRepository;
        $ocorrencia->setNrorg($params['ROW']['NRORG']);
        $ocorrencia->setNrtipoprogocorrencia($params['ROW']['NRTIPOPROGOCORRENCIA']);
        $ocorrencia->setNrsitufrequencia($params['ROW']['NRSITUFREQUENCIA']);
        $ocorrencia->setNrprogocorrencia($params['ROW']['NRPROGOCORRENCIA']);
        $ocorrencia->setNrvinculom($params['VINCULO']);
        $ocorrencia->setDtinicioprogocor(DateUtil::getDataDeString(substr($params['ROW']['DTINICIOPROGOCOR'], 0,10), DateUtil::FORMATO_DATA, true));
        $ocorrencia->setDtfimprogocor(DateUtil::getDataDeString(substr($params['ROW']['DTFIMPROGOCOR'], 0,10), DateUtil::FORMATO_DATA, true));
        $ocorrencia->setIdativo('S');
        $ocorrencia->setIdtipodiacalendario($params['ROW']['IDTIPODIACALENDARIO']);
        $ocorrencia->setIddiassemana($params['ROW']['IDDIASSEMANA']);
        $ocorrencia->setIdtipohorario($params['ROW']['IDTIPOHORARIO']);
        $ocorrencia->setNrmotivoocorfreq($params['ROW']['NRMOTIVOOCORFREQ']);
        $ocorrencia->setNrhordiam($params['ROW']['NRHORDIAM']);
        
        // Tratativas para salvar quantidades. Se vazios, devem ser nulos ao inves de 0
        if (!empty($params['ROW']['QTMINGATILHOPROGOCOR'])) {
            $ocorrencia->setQtmingatilhoprogocor($params['ROW']['QTMINGATILHOPROGOCOR']);
        }
        if (!empty($params['ROW']['QTMAXGATILHOPROGOCOR'])) {
            $ocorrencia->setQtmaxgatilhoprogocor($params['ROW']['QTMAXGATILHOPROGOCOR']);
        }
        if (!empty($params['ROW']['QTMINIMAPROGOCOR'])) {
            $ocorrencia->setQtminimaprogocor($params['ROW']['QTMINIMAPROGOCOR']);
        }
        if (!empty($params['ROW']['QTMAXIMAPROGOCOR'])) {
            $ocorrencia->setQtmaximaprogocor($params['ROW']['QTMAXIMAPROGOCOR']);
        }
        
        $ocorrencia->setVrcoeficienteacrescimo($params['ROW']['VRCOEFICIENTEACRESCIMO']);
        $ocorrencia->setDsobservacao($params['ROW']['DSOBSERVACAO']);
        $ocorrencia->setNrmarcacaoponto($params['ROW']['NRMARCACAOPONTO']);
        $ocorrencia->setNrintervalohorario($params['ROW']['NRINTERVALOHORARIO']);
        $ocorrencia->setIdprogocorconfirmada($params['ROW']['IDPROGOCORCONFIRMADA']);
        $ocorrencia->setDtinclusao( DateUtil::getDataDeString(substr($params['ROW']['DTINCLUSAO'], 0,10)) );
        $ocorrencia->setCdoperinclusao($params['ROW']['CDOPERINCLUSAO']);
        $ocorrencia->setNrorginclusao($params['ROW']['NRORGINCLUSAO']);
        $ocorrencia->setDtultatu( DateUtil::getDataDeString(substr($params['ROW']['DTULTATU'], 0,10)) );
        $ocorrencia->setCdoperultatu($params['ROW']['CDOPERULTATU']);
        $ocorrencia->setNrorgultatu($params['ROW']['NRORGULTATU']);
        $ocorrencia->setNrsitufreqgatilho($params['ROW']['NRSITUFREQGATILHO']);
        $ocorrencia->setCdtabecdi($params['ROW']['CDTABECDI']);
        $ocorrencia->setCddiagnost($params['ROW']['CDDIAGNOST']);
        $ocorrencia->setNrpessoa($params['ROW']['NRPESSOA']);
        $this->entityManager->persist($ocorrencia);
        $this->entityManager->flush($ocorrencia);
    }
    
    public function atualizaFeriado(
        $nrVinculom     = null,
        $nrEstrutLegal  = null,
        $nrCalendario   = null
    ) {
        
        $status = $this->ocorrenciaFrequenciaRepository->atualizaFeriado(
            $this->hcmParameters->getNrorgTrabalho(),
            $this->hcmParameters->getDtCompetencia(),
            $nrVinculom,
            $nrEstrutLegal,
            $nrCalendario
        );
        
        Util::updateNovoCodigo('PTO_PROGOCORRENCIA');
        
        return $status;
    }
    
    /**
     *  Rotina capaz de indicar se um determinado vinculo ou estrutura legal esta associado um determinado feriado
     * 
     *  @param  $dataCalendario     = null
     *  @param  $nrVinculom         = null
     *  @param  $nrEstrutLegal      = null
     * 
     */
    public function isDiaFeriadoCalendario(
        $dataCalendario     = null,
        $nrVinculom         = null,
        $nrEstrutLegal      = null
    ) {
        $ocorCalendario = $this->ocorrenciaFrequenciaRepository->getDiasCalendarioFeriado(
            $this->hcmParameters->getNrorgTrabalho(),
            $this->hcmParameters->getDtCompetencia(),
            $dataCalendario,
            $nrVinculom,
            $nrEstrutLegal
        );
        
        return !empty($ocorCalendario);
    }
    
    /**
     *  Rotina capaz de informar o valor do campo IDTIPOCALENDARIO de uma ocorrencia verificando a existencia de feriados para o dia informado
     * 
     *  @param  $dataCalendario     = null
     *  @param  $nrVinculom         = null
     *  @param  $nrEstrutLegal      = null
     * 
     */   
    public function getIdtipocalendarioProgocor(
        $dataCalendario,
        $nrVinculom,
        $nrEstrutLegal      = null
    ) {
        $isFeriado = $this->isDiaFeriadoCalendario(
            $dataCalendario,
            $nrVinculom,
            $nrEstrutLegal
        );
        
        return $isFeriado ? self::IDTIPODIACALENDARIO_FERIADO : self::IDTIPODIACALENDARIO_NORMAL;
    }
}