/*
479244 - GET_SITUACAO_FREQUENCIA_PERIODO.SQL

Marcao de Espelho Ponto
06/06/2024

Funo: get_situacao_frequencia_periodo
Descrio: Fornece um conjunto de dados estruturado com um resumo detalhado da situao de frequncia de funcionrios
    dentro de um perodo de apurao especfico. Ela busca informaes como faltas, horas extras, atrasos, adicionais
    noturnos, salrios, entre outros, a partir de diversas tabelas do sistema. Para integrao direta em consultas SQL
    no Zmart BI.
Retorno: Pipeline (Tabela Aninhada).
Parmetros:
    - p_nrorg:          Nmero da organizao.
    - p_nrorg_padrao:   Nmero da organizao padro (utilizado caso algumas configuraes no sejam encontradas na
        organizao especfica).
    - p_dtinicio:       Data de incio do perodo de apurao (formato string).
    - p_dtfim:          Data de fim do perodo de apurao (formato string).
Observaes:
	- As datas de inicio e fim precisam corresponder ao perodo praticado pelo cliente, caso contrrio no ser
        retornado dados.
	- Cada linha representa um dia de trabalho de um funcionrio e seus respectivos registros de ponto.
	- Colunas:
		- nrvinculom: Nmero de identificao do vnculo empregatcio do funcionrio.
        - nmvinculom: Nome do funcionrio.
        - nrperiodoapuracao: Identificador do perodo de apurao ao qual os dados se referem.
        - nrestrutlegal: Cdigo da estrutura legal  qual o funcionrio pertence.
        - nmestrutlegal: Descrio da estrutura legal (nome e cdigo).
        - nrestrutgeren: Cdigo da estrutura gerencial  qual o funcionrio pertence.
        - nmestrutgeren: Descrio da estrutura gerencial (nome e cdigo).
        - saldo_folgar_per_bhr_norm: Saldo de folgas do funcionrio no perodo, sem considerar acrscimos, em horas
            padro (normalmente 44 horas semanais).
        - saldo_folgar_acr_per_bhr_norm: Saldo de folgas do funcionrio no perodo, considerando acrscimos, em horas
            padro.
        - saldo_folgar_acr_per_bhr_25: Saldo de folgas do funcionrio no perodo, considerando acrscimos, em horas com
            adicional de 25% (normalmente 55 horas semanais).
        - dsr_descontado: Quantidade de Descansos Semanais Remunerados (DSRs) descontados no perodo.
        - faltas_diurnas: Quantidade de faltas diurnas no perodo.
        - faltas_noturnas: Quantidade de faltas noturnas no perodo.
        - ex_normal_np: Total de horas extras normais no pagas no perodo.
        - ex_normal: Total de horas extras normais pagas no perodo.
        - ex_dia_feriad: Total de horas extras em dias de feriado no perodo.
        - ex_not_feriad: Total de horas extras noturnas em feriados no perodo.
        - atrasos_diurnos: Total de atrasos diurnos no perodo.
        - adicional_noturno: Total de adicional noturno pago no perodo.
        - vrsalario: Valor do salrio do funcionrio.
        - dtadmissaovinc: Data de admisso do funcionrio.
        - dstpeventoctbl: Descrio do tipo de evento da folha de pagamento associado  estrutura gerencial do
            funcionrio.
        - horas_banco_a_pagar: Valor das horas no banco de horas que devem ser pagas ao funcionrio.
        - reflexo: Valor do reflexo das horas do banco de horas em dias de folga.
*/

-- Remove todas as depedncias
DROP FUNCTION get_situacao_frequencia_periodo;
/

DROP TYPE situacao_frequencia_periodo_tab;
/

-- Tipos
CREATE OR REPLACE TYPE situacao_frequencia_periodo_row AS OBJECT (
    nrvinculom                      NUMBER,
    nmvinculom                      VARCHAR2(100),
    nrperiodoapuracao               NUMBER,
    nrestrutlegal                   NUMBER,
    nmestrutlegal                   VARCHAR2(120),
    nrestrutgeren                   NUMBER,
    nmestrutgeren                   VARCHAR2(120),
    saldo_folgar_per_bhr_norm       NUMBER,
    saldo_folgar_acr_per_bhr_norm   NUMBER,
    saldo_folgar_acr_per_bhr_25     NUMBER,
    dsr_descontado                  VARCHAR2(4),
    faltas_diurnas                  VARCHAR2(4),
    faltas_noturnas                 VARCHAR2(4),
    ex_normal_np                    NUMBER,
    ex_normal                       NUMBER,
    ex_dia_feriad                   NUMBER,
    ex_not_feriad                   NUMBER,
    atrasos_diurnos                 NUMBER,
    adicional_noturno               NUMBER,
    vrsalario                       NUMBER,
    dtadmissaovinc                  VARCHAR2(10),
    dstpeventoctbl                  VARCHAR2(50),
    horas_banco_a_pagar             NUMBER,
    reflexo                         NUMBER
);
/

CREATE OR REPLACE TYPE situacao_frequencia_periodo_tab AS TABLE OF situacao_frequencia_periodo_row;
/

-- Funo
CREATE OR REPLACE FUNCTION get_situacao_frequencia_periodo (
    p_nrorg        IN NUMBER,
    p_nrorg_padrao IN NUMBER,
    p_dtinicio     IN VARCHAR2,
    p_dtfim        IN VARCHAR2
) RETURN situacao_frequencia_periodo_tab PIPELINED
IS
    v_row situacao_frequencia_periodo_row := situacao_frequencia_periodo_row(
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL );
BEGIN
    FOR registro IN (
        WITH
        periodo_apuracao AS (
            SELECT
                pa.nrperiodoapuracao,
                pa.dtinicialapuracao,
                pa.dtfinalapuracao
            FROM pto_periodoapuracao pa
            WHERE pa.nrorg                 = p_nrorg
              AND pa.dtinicialapuracao     = TO_DATE( p_dtinicio, 'DD/MM/YYYY' )
              AND pa.dtfinalapuracao       = TO_DATE( p_dtfim,    'DD/MM/YYYY' )
              AND pa.nrtipoperiodoapuracao = ( SELECT nrtipoperiodoapuracao
                                               FROM pto_tipoperiodoapuracao
                                               WHERE nrorg          = pa.nrorg
                                                 AND nrtipovinculom = 1 )
        ),
        vinculos AS ( 
            SELECT
                vm.nrorg,
                vm.nrvinculom,
                vm.nmvinculom,
                vh.nrestrutlegal,
                vh.nrestrutgeren,
                vh.nrescalatrabm,
                legal.nmestruturam AS nmlegal,
                gerencial.nmestruturam AS nmgerencial,
                estrut_calendh.nrcalendario,
                pb.vrcoeficienteacrescimo,
                pa.nrperiodoapuracao,
                pa.dtinicialapuracao,
                pa.dtfinalapuracao,
                vm.dtadmissaovinc,
                TRUNC( greatest( pa.dtinicialapuracao, vm.dtadmissaovinc ) )                       AS dtinicialapuracao_tratada,
                TRUNC( least( pa.dtfinalapuracao, nvl( vm.dtrescisaovinc, pa.dtfinalapuracao ) ) ) AS dtfinalapuracao_tratada,
                nvl( eh.qthrsemesctrabh, 0 ) * 5                                                   AS total_jornada,
                evh.vrpercentualevento                                                             AS vrpercentual_evento_a_pagar,
                ae.nraltesalario,
                ae.vrsalario,
                tpe.dstpeventoctbl
            FROM gpe_vinculom vm
            JOIN gpe_vinculoh vh ON vh.nrvinculom     = vm.nrvinculom
                                AND vh.nrorg          = vm.nrorg
                                AND vh.dtmescompetenc = ( SELECT dtmescompetenc FROM ( SELECT dtmescompetenc
                                                          FROM gpe_vinculoh
                                                          WHERE nrorg      = vh.nrorg
                                                            AND nrvinculom = vh.nrvinculom
                                                            AND dtmescompetenc <= TRUNC( TO_DATE( p_dtfim, 'DD/MM/YYYY' ), 'MM' )
                                                          ORDER BY dtmescompetenc DESC ) WHERE ROWNUM = 1 )
            JOIN periodo_apuracao pa ON 1 = 1
            JOIN estruturam legal ON legal.nrestruturam = vh.nrestrutlegal
                                 AND legal.nrorg        = vm.nrorg
            LEFT JOIN estruturam gerencial ON gerencial.nrestruturam = vh.nrestrutgeren
                                          AND gerencial.nrorg        = vm.nrorg
            JOIN estruturam estrut_calend ON estrut_calend.nrorg = vm.nrorg
                                         AND estrut_calend.nrestruturam = nvl( ( 
                    SELECT mov.nrestruturam
                    FROM gpe_movimentacao mov
                    JOIN gpe_paramorg po ON po.nrorg = mov.nrorg
                        AND po.nrtpestrutcalend IS NOT NULL
                    WHERE mov.nrorg = estrut_calend.nrorg
                        AND mov.nrvinculom = vm.nrvinculom
                        AND mov.dtinimoviment <= last_day( pa.dtfinalapuracao )
                        AND ( mov.dtfimmoviment IS NULL OR mov.dtfimmoviment > last_day( pa.dtfinalapuracao ) )
                        AND mov.nrtipoestrutura = po.nrtpestrutcalend
                    ORDER BY mov.dtinimoviment DESC
                    FETCH FIRST ROW ONLY
                     ), vh.nrestrutlegal )
            LEFT JOIN estruturah estrut_calendh ON estrut_calendh.nrorg = vm.nrorg AND estrut_calendh.nrestruturam = estrut_calend.nrestruturam
                AND estrut_calendh.dtmescompetenc = ( SELECT MAX( dtmescompetenc )
                                                      FROM estruturah
                                                      WHERE nrorg = estrut_calendh.nrorg
                                                        AND nrestruturam = estrut_calendh.nrestruturam
                                                        AND dtmescompetenc <= pa.dtfinalapuracao )
            LEFT JOIN fpa_tpeventoctbl tpe ON tpe.nrtpeventoctbl = gerencial.nrtpeventoctbl
                AND ( tpe.nrorg = gerencial.nrorg OR tpe.nrorg = 0 )
            LEFT JOIN pto_altebancohoras ab ON ab.nrorg = vm.nrorg AND ab.nrvinculom = vm.nrvinculom
                AND ab.dtinicioaltebhoras = ( SELECT MAX( dtinicioaltebhoras )
                                              FROM pto_altebancohoras
                                              WHERE nrorg = ab.nrorg
                                                AND nrvinculom = ab.nrvinculom
                                                AND ( dtinicioaltebhoras <= pa.dtfinalapuracao AND nvl( dtfimaltebhoras, pa.dtinicialapuracao ) >= pa.dtinicialapuracao ) )
            LEFT JOIN pto_bancohorasprog pb ON pb.nrorg = vm.nrorg AND pb.nrbancohoras = ab.nrbancohoras
                AND pb.nrtipoprogocorrencia = 12
                AND pb.idativo = 'S'
                AND ( pb.dtinicioprogbhoras <= pa.dtfinalapuracao AND nvl( pb.dtfimprogbhoras, pa.dtinicialapuracao ) >= pa.dtinicialapuracao )
            LEFT JOIN pto_assocsitufreq asf_quita_banco
                ON asf_quita_banco.nrcalculositufreq = 8000
                AND asf_quita_banco.nrorg = ( 
                    SELECT nrorg
                    FROM pto_assocsitufreq
                    WHERE nrcalculositufreq = asf_quita_banco.nrcalculositufreq
                        AND ( nrorg = p_nrorg OR nrorg = p_nrorg_padrao )
                    GROUP BY nrorg
                    ORDER BY ( 
                        CASE
                            WHEN nrorg = p_nrorg AND COUNT( nrorg ) = 1 THEN -1
                            ELSE nrorg
                        END
                     )
                    FETCH FIRST ROW ONLY
                 )
                AND asf_quita_banco.nrassocsitufreq = ( SELECT MAX( nrassocsitufreq )
                                                        FROM pto_assocsitufreq
                                                        WHERE nrorg = asf_quita_banco.nrorg
                                                          AND nrcalculositufreq = asf_quita_banco.nrcalculositufreq )
            LEFT JOIN pto_situfreqparam sfp_quita_banco ON ( sfp_quita_banco.nrorg = vm.nrorg OR sfp_quita_banco.nrorg = p_nrorg_padrao )
                                                       AND sfp_quita_banco.nrsitufrequencia = asf_quita_banco.nrsitufrequencia
            JOIN fpa_eventoh evh ON ( evh.nrorg = vm.nrorg OR evh.nrorg = p_nrorg_padrao )
                AND evh.nreventom = nvl( sfp_quita_banco.nreventom, 10 )
                AND evh.dtmescompetenc = ( SELECT MAX( dtmescompetenc )
                                           FROM fpa_eventoh
                                           WHERE nrorg = evh.nrorg
                                             AND nreventom = evh.nreventom )
            LEFT JOIN gpe_altesalario ae ON ae.nrorg = vm.nrorg
                                        AND ae.nrvinculom = vm.nrvinculom
                                        AND ae.dtaltesalario = ( SELECT MAX( dtaltesalario )
                                                                 FROM gpe_altesalario
                                                                 WHERE nrorg = ae.nrorg
                                                                   AND nrvinculom = ae.nrvinculom )
            LEFT JOIN gpe_escalatrabh eh ON eh.nrorg = vm.nrorg
                AND eh.nrescalatrabm = vh.nrescalatrabm
                AND eh.dtmescompetenc = ( SELECT MAX( dtmescompetenc )
                                          FROM gpe_escalatrabh
                                          WHERE nrorg = eh.nrorg
                                            AND nrescalatrabm = eh.nrescalatrabm )
            WHERE vm.nrorg = p_nrorg
                AND EXISTS ( 
                    SELECT 1
                    FROM pto_resumositufreq
                    WHERE nrorg = vm.nrorg
                        AND nrvinculom = vm.nrvinculom
                        AND nrperiodoapuracao = pa.nrperiodoapuracao
                 )
                AND vm.nrtipovinculom = 1
         ),
        total_resumos AS ( 
            SELECT
                asf.nrcalculositufreq,
                rsf.nrvinculom,
                SUM( rsf.qtrefapursitufreq ) total
            FROM pto_resumositufreq rsf
            LEFT JOIN pto_assocsitufreq asf
                ON asf.nrsitufrequencia = rsf.nrsitufrequencia
                AND asf.nrorg = ( 
                    SELECT nrorg
                    FROM pto_assocsitufreq
                    WHERE nrsitufrequencia = asf.nrsitufrequencia
                        AND ( nrorg = p_nrorg OR nrorg = p_nrorg_padrao )
                    GROUP BY nrorg
                    ORDER BY ( 
                        CASE
                            WHEN nrorg = p_nrorg AND COUNT( nrorg ) = 1 THEN -1
                            ELSE nrorg
                        END
                     )
                    FETCH FIRST ROW ONLY
                 )
                AND asf.nrassocsitufreq = ( SELECT MAX( nrassocsitufreq )
                                            FROM pto_assocsitufreq
                                            WHERE nrorg = asf.nrorg
                                                AND nrsitufrequencia = asf.nrsitufrequencia )
            WHERE rsf.nrorg = p_nrorg
                AND rsf.nrperiodoapuracao IN ( SELECT nrperiodoapuracao FROM periodo_apuracao )
                AND rsf.nrvinculom IN ( SELECT nrvinculom FROM vinculos )
            GROUP BY asf.nrcalculositufreq, rsf.nrvinculom
         ),
        total_extra_aut AS ( 
            SELECT
                nrvinculom,
                SUM( total ) valor
            FROM total_resumos
            WHERE nrcalculositufreq IN ( 
                2200, 2300, 2900, 3000,
                3700, 3800, 4400, 4500,
                5200, 5300, 5900, 6000,
                6700, 6800, 7400, 7500
             )
            GROUP BY nrvinculom
         ),
        total_extra_pnd AS ( 
            SELECT
                nrvinculom,
                SUM( total ) valor
            FROM total_resumos
            WHERE nrcalculositufreq IN ( 
                2000, 2100, 2600, 2700,
                3500, 3600, 4100, 4200,
                5000, 5100, 5600, 5700,
                6500, 6600, 7100, 7200
             )
            GROUP BY nrvinculom
         ),
        total_extra_fer_dia AS ( 
            SELECT
                nrvinculom,
                SUM( total ) valor
            FROM total_resumos
            WHERE nrcalculositufreq IN ( 
                6700, 6800
             )
            GROUP BY nrvinculom
         ),
        total_extra_fer_not AS ( 
            SELECT
                nrvinculom,
                SUM( total ) valor
            FROM total_resumos
            WHERE nrcalculositufreq IN ( 
                7400, 7500
             )
            GROUP BY nrvinculom
         ),
        total_adnot AS ( 
            SELECT
                nrvinculom,
                SUM( total ) valor
            FROM total_resumos
            WHERE nrcalculositufreq IN ( 9000 )
            GROUP BY nrvinculom
         ),
        total_atras AS ( 
            SELECT
                nrvinculom,
                SUM( total ) valor
            FROM total_resumos
            WHERE nrcalculositufreq IN ( 
                10100, 10200
             )
            GROUP BY nrvinculom
         )
        SELECT
            pto_resumositufreq.nrvinculom,
            pto_resumositufreq.nmvinculom,
            pto_resumositufreq.nrperiodoapuracao,
            pto_resumositufreq.nrestrutlegal,
            pto_resumositufreq.nmlegal || ' - ' || pto_resumositufreq.nrestrutlegal AS nmestrutlegal,
            pto_resumositufreq.nrestrutgeren,
            pto_resumositufreq.nmgerencial || ' - ' || pto_resumositufreq.nrestrutgeren AS nmestrutgeren,
            DECODE( pto_resumositufreq.saldo_folgar_per_bhr_norm,     NULL, '0000', pto_resumositufreq.saldo_folgar_per_bhr_norm ) AS saldo_folgar_per_bhr_norm,
            DECODE( pto_resumositufreq.saldo_folgar_acr_per_bhr_norm, NULL, '0000', pto_resumositufreq.saldo_folgar_acr_per_bhr_norm ) AS saldo_folgar_acr_per_bhr_norm,
            DECODE( pto_resumositufreq.saldo_folgar_acr_per_bhr_norm, NULL, '0000',
                pto_resumositufreq.saldo_folgar_acr_per_bhr_norm / ( 1 + ( nvl( vrcoeficienteacrescimo, 0 ) / 100 ) )  ) AS saldo_folgar_acr_per_bhr_25,
            DECODE( pto_resumositufreq.dsr_descontado,                NULL, '0000', pto_resumositufreq.dsr_descontado ) AS dsr_descontado,
            DECODE( pto_resumositufreq.faltas_diurnas,                NULL, '0000', pto_resumositufreq.faltas_diurnas ) AS faltas_diurnas,
            DECODE( pto_resumositufreq.faltas_noturnas,               NULL, '0000', pto_resumositufreq.faltas_noturnas ) AS faltas_noturnas,
            pto_resumositufreq.t_extra_pnd AS ex_normal_np,
            pto_resumositufreq.t_extra_aut AS ex_normal,
            pto_resumositufreq.t_extra_fer_dia AS ex_dia_feriad,
            pto_resumositufreq.t_extra_fer_not AS ex_not_feriad,
            pto_resumositufreq.t_atras AS atrasos_diurnos,
            pto_resumositufreq.t_adnot AS adicional_noturno,
            pto_resumositufreq.vrsalario,
            substr( pto_resumositufreq.dtadmissaovinc, 0, 10 ) AS dtadmissaovinc,
            pto_resumositufreq.dstpeventoctbl,
            round( 
                ( greatest( pto_resumositufreq.saldo_folgar_acr_per_bhr_norm, 0 ) / razao_acresc_saldo ) * ( ( pto_resumositufreq.vrsalario / greatest( pto_resumositufreq.total_jornada, 1 ) ) * razao_acresc_even )
            , 2 ) AS horas_banco_a_pagar,
            round( 
                ( round( 
                ( greatest( pto_resumositufreq.saldo_folgar_acr_per_bhr_norm, 0 ) / razao_acresc_saldo ) * ( ( pto_resumositufreq.vrsalario / greatest( pto_resumositufreq.total_jornada, 1 ) ) * razao_acresc_even )
                , 2 ) / greatest( pto_resumositufreq.dias_periodo - pto_resumositufreq.dias_folga_reflexo, 1 ) ) * pto_resumositufreq.dias_folga_reflexo
            , 2 ) AS reflexo
        FROM ( SELECT vinculos.nrvinculom,
                vinculos.nmvinculom,
                vinculos.nrorg,
                vinculos.nrperiodoapuracao,
                vinculos.nrestrutlegal,
                vinculos.nrestrutgeren,
                vinculos.nrescalatrabm,
                vinculos.nmlegal,
                vinculos.nmgerencial,
                vinculos.nrcalendario,
                vinculos.vrcoeficienteacrescimo,
                ( 1+( nvl( vinculos.vrcoeficienteacrescimo, 0 )/100 ) ) razao_acresc_saldo,
                vinculos.dtinicialapuracao,
                vinculos.dtfinalapuracao,
                vinculos.total_jornada,
                vinculos.vrpercentual_evento_a_pagar,
                ( 1+( vinculos.vrpercentual_evento_a_pagar/100 ) ) razao_acresc_even,
                vinculos.vrsalario,
                vinculos.dtadmissaovinc,
                vinculos.dstpeventoctbl,
                nvl( total_extra_aut.valor, 0 ) t_extra_aut,
                nvl( total_extra_pnd.valor, 0 ) t_extra_pnd,
                nvl( total_adnot.valor, 0 ) t_adnot,
                nvl( total_atras.valor, 0 ) t_atras,
                nvl( total_extra_fer_dia.valor, 0 ) t_extra_fer_dia,
                nvl( total_extra_fer_not.valor, 0 ) t_extra_fer_not,
                ( SELECT nvl( round( qtrefapursitufreq, 7 ), '0000' )
                    FROM ( SELECT qtrefapursitufreq
                            FROM pto_resumositufreq RS
                            JOIN pto_assocsitufreq asf ON asf.nrsitufrequencia = RS.nrsitufrequencia
                                                    AND asf.nrcalculositufreq = 18020
                            WHERE RS.nrvinculom        = vinculos.nrvinculom
                            AND RS.nrperiodoapuracao IN ( SELECT nrperiodoapuracao FROM periodo_apuracao )
                            AND RS.nrorg             = p_nrorg
                            AND RS.dtapuracaositufreq BETWEEN vinculos.dtinicialapuracao AND vinculos.dtfinalapuracao
                        ORDER BY dtapuracaositufreq DESC )
                    WHERE ROWNUM = 1 ) AS saldo_folgar_per_bhr_norm,
                ( SELECT nvl( round( qtrefapursitufreq, 7 ), '0000' )
                    FROM ( SELECT qtrefapursitufreq
                            FROM pto_resumositufreq RS
                            JOIN pto_assocsitufreq asf ON asf.nrsitufrequencia = RS.nrsitufrequencia
                                                    AND asf.nrcalculositufreq = 18420
                            WHERE RS.nrvinculom        = vinculos.nrvinculom
                            AND RS.nrperiodoapuracao IN ( SELECT nrperiodoapuracao FROM periodo_apuracao )
                            AND RS.nrorg             = p_nrorg
                            AND RS.dtapuracaositufreq BETWEEN vinculos.dtinicialapuracao AND vinculos.dtfinalapuracao
                        ORDER BY dtapuracaositufreq DESC )
                    WHERE ROWNUM = 1 ) AS saldo_folgar_acr_per_bhr_norm,
                ( SELECT COUNT( qtrefapursitufreq )
                    FROM pto_resumositufreq RS
                    JOIN pto_assocsitufreq asf ON asf.nrsitufrequencia = RS.nrsitufrequencia
                                            AND asf.nrcalculositufreq = 12100
                    WHERE RS.nrvinculom        = vinculos.nrvinculom
                    AND RS.nrperiodoapuracao IN ( SELECT nrperiodoapuracao FROM periodo_apuracao )
                    AND RS.nrorg             = p_nrorg
                    AND RS.dtapuracaositufreq BETWEEN vinculos.dtinicialapuracao AND vinculos.dtfinalapuracao ) AS dsr_descontado,
                ( SELECT COUNT( qtrefapursitufreq )
                    FROM pto_resumositufreq RS
                    JOIN pto_assocsitufreq asf ON asf.nrsitufrequencia = RS.nrsitufrequencia
                                            AND asf.nrcalculositufreq = 10500
                    WHERE RS.nrvinculom        = vinculos.nrvinculom
                    AND RS.nrperiodoapuracao IN ( SELECT nrperiodoapuracao FROM periodo_apuracao )
                    AND RS.nrorg             = p_nrorg
                    AND RS.dtapuracaositufreq BETWEEN vinculos.dtinicialapuracao AND vinculos.dtfinalapuracao ) AS faltas_diurnas,
                ( SELECT COUNT( qtrefapursitufreq )
                    FROM pto_resumositufreq RS
                    JOIN pto_assocsitufreq asf ON asf.nrsitufrequencia = RS.nrsitufrequencia
                                            AND asf.nrcalculositufreq = 10600
                    WHERE RS.nrvinculom        = vinculos.nrvinculom
                    AND RS.nrperiodoapuracao IN ( SELECT nrperiodoapuracao FROM periodo_apuracao )
                    AND RS.nrorg             = p_nrorg
                    AND RS.dtapuracaositufreq BETWEEN vinculos.dtinicialapuracao AND vinculos.dtfinalapuracao ) AS faltas_noturnas,
                ( SELECT COUNT( oc.dtcalendario ) + ( TRUNC( vinculos.dtfinalapuracao_tratada+1, 'IW' ) - TRUNC( vinculos.dtinicialapuracao_tratada, 'IW' ) ) / 7 /* Quantidade de domingos no periodo */
                    FROM ocorcalendario oc
                    WHERE oc.nrorg = p_nrorg
                    AND oc.nrcalendario = vinculos.nrcalendario
                    AND oc.dtcalendario BETWEEN vinculos.dtinicialapuracao_tratada AND vinculos.dtfinalapuracao_tratada
                     ) AS dias_folga_reflexo,
                    greatest( vinculos.dtfinalapuracao_tratada - vinculos.dtinicialapuracao_tratada + 1, 0 ) dias_periodo
            FROM vinculos
            LEFT JOIN total_extra_aut ON total_extra_aut.nrvinculom = vinculos.nrvinculom
            LEFT JOIN total_extra_pnd ON total_extra_pnd.nrvinculom = vinculos.nrvinculom
            LEFT JOIN total_adnot ON total_adnot.nrvinculom = vinculos.nrvinculom
            LEFT JOIN total_atras ON total_atras.nrvinculom = vinculos.nrvinculom
            LEFT JOIN total_extra_fer_dia ON total_extra_fer_dia.nrvinculom = vinculos.nrvinculom
            LEFT JOIN total_extra_fer_not ON total_extra_fer_not.nrvinculom = vinculos.nrvinculom
         ) pto_resumositufreq
        ORDER BY pto_resumositufreq.nrvinculom
    ) LOOP
        v_row.nrvinculom 					:= registro.nrvinculom;
		v_row.nmvinculom 					:= registro.nmvinculom;
		v_row.nrperiodoapuracao 			:= registro.nrperiodoapuracao;
		v_row.nrestrutlegal 				:= registro.nrestrutlegal;
		v_row.nmestrutlegal 			    := registro.nmestrutlegal;
		v_row.nrestrutgeren 				:= registro.nrestrutgeren;
		v_row.nmestrutgeren 			    := registro.nmestrutgeren;
		v_row.saldo_folgar_per_bhr_norm 	:= registro.saldo_folgar_per_bhr_norm;
		v_row.saldo_folgar_acr_per_bhr_norm	:= registro.saldo_folgar_acr_per_bhr_norm;
		v_row.saldo_folgar_acr_per_bhr_25 	:= registro.saldo_folgar_acr_per_bhr_25;
		v_row.dsr_descontado 				:= registro.dsr_descontado;
		v_row.faltas_diurnas 				:= registro.faltas_diurnas;
		v_row.faltas_noturnas 				:= registro.faltas_noturnas;
		v_row.ex_normal_np 					:= registro.ex_normal_np;
		v_row.ex_normal 					:= registro.ex_normal;
		v_row.ex_dia_feriad		 			:= registro.ex_dia_feriad;
		v_row.ex_not_feriad 				:= registro.ex_not_feriad;
		v_row.atrasos_diurnos 				:= registro.atrasos_diurnos;
		v_row.adicional_noturno 			:= registro.adicional_noturno;
		v_row.vrsalario 					:= registro.vrsalario;
		v_row.dtadmissaovinc 				:= registro.dtadmissaovinc;
		v_row.dstpeventoctbl 				:= registro.dstpeventoctbl;
		v_row.horas_banco_a_pagar 			:= registro.horas_banco_a_pagar;
		v_row.reflexo 						:= registro.reflexo;
        PIPE ROW( v_row );
    END LOOP;
END;
/

--Com preenchimento direto dos parmetros
--SELECT * FROM TABLE( get_situacao_frequencia_periodo( 3260, 0, '16/01/2024', '15/02/2024' ) );

--Com os parmetros apenas
--SELECT * FROM TABLE( get_situacao_frequencia_periodo( :p_nrorg, :p_nrorg_padrao, :p_dtinicio, :p_dtfim ) );
/
